@@ -32,14 +32,16 @@ pjoin = os.path.join
 # Constants
 #-----------------------------------------------------------------------------
 
-bundled_version = (4, 3, 1)
+bundled_version = (4, 3, 3)
 vs = '%i.%i.%i' % bundled_version
 libzmq = "zeromq-%s.tar.gz" % vs
 libzmq_url = "https://github.com/zeromq/libzmq/releases/download/v{vs}/{libzmq}".format(
     vs=vs,
     libzmq=libzmq,
 )
-libzmq_checksum = "sha1:6cce22d830eaf95feff7cab00744df13ad7ab7f3"
+libzmq_checksum = (
+    "sha256:9d9285db37ae942ed0780c016da87060497877af45094ff9e1a1ca736e3875a2"
+)
 
 HERE = os.path.dirname(__file__)
 ROOT = os.path.dirname(HERE)
@@ -126,12 +128,22 @@ def stage_platform_hpp(zmqroot):
         platform_dir = pjoin(HERE, 'include_win32')
     else:
         info("attempting ./configure to generate platform.hpp")
-
-        p = Popen('./configure', cwd=zmqroot, shell=True,
-            stdout=PIPE, stderr=PIPE,
-        )
-        o,e = p.communicate()
-        if p.returncode:
+        failed = False
+        try:
+            p = Popen(
+                ["./configure", "--disable-drafts"],
+                cwd=zmqroot,
+                stdout=PIPE,
+                stderr=PIPE,
+            )
+        except OSError as err:
+            failed = True
+            e = str(err)
+        else:
+            o, e = p.communicate()
+            e = e.decode("utf8", "replace")
+            failed = bool(p.returncode)
+        if failed:
             warn("failed to configure libzmq:\n%s" % e)
             if sys.platform == 'darwin':
                 platform_dir = pjoin(HERE, 'include_darwin')
