@@ -21,6 +21,7 @@ if __name__ == '__main__':
     docwriter.package_skip_patterns += [
         r'\.tests$',
         r'\.backend$',
+        r'\.auth$',
         r'\.eventloop\.minitornado$',
         r'\.green\.eventloop$',
         r'\.sugar$',
@@ -29,6 +30,7 @@ if __name__ == '__main__':
 
     docwriter.module_skip_patterns += [
         r'\.eventloop\.stack_context$',
+        r'\.eventloop\.future$',
         r'\.error$',
         r'\.green\..+$',
         r'\.utils\.initthreads$',
@@ -36,6 +38,7 @@ if __name__ == '__main__':
         r'\.utils\.garbage$',
         r'\.utils\.rebuffer$',
         r'\.utils\.strtypes$',
+        r'\.zmq$',
         ]
     
     # Now, generate the outputs
