@@ -23,6 +23,7 @@ import re
 import shutil
 from subprocess import check_output
 import sys
+import time
 
 from contextlib import contextmanager
 
@@ -48,6 +49,7 @@ if not sdkroot:
         os.environ["SDKROOT"] = sdkroot
     else:
         print("SDK not found at %r" % sdkroot)
+        time.sleep(10)
 
 # Workaround for PyPy3 5.8
 if 'LDFLAGS' not in os.environ:
@@ -57,21 +59,26 @@ if 'LDFLAGS' not in os.environ:
 if 'MACOSX_DEPLOYMENT_TARGET' not in os.environ:
     os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.9'
 
+# set compiler env (avoids issues with missing 'gcc-4.2' on py27, etc.)
+if 'CC' not in os.environ:
+    os.environ['CC'] = 'clang'
+
+if 'CXX' not in os.environ:
+    os.environ['CXX'] = 'clang++'
+
 _framework_py = lambda xy: "/Library/Frameworks/Python.framework/Versions/{0}/bin/python{0}".format(xy)
 py_exes = {
-    '3.8' : _framework_py('3.8'),
-    '3.7' : _framework_py('3.7'),
-    '2.7' : _framework_py('2.7'),
-    '3.5' : _framework_py('3.5'),
-    '3.6' : _framework_py('3.6'),
-    'pypy': "/usr/local/bin/pypy",
-    'pypy3': "/usr/local/bin/pypy3",
+    "3.9": _framework_py("3.9"),
+    "3.8": _framework_py("3.8"),
+    "3.7": _framework_py("3.7"),
+    "3.5": _framework_py("3.5"),
+    "3.6": _framework_py("3.6"),
+    "pypy3": "/usr/local/bin/pypy3",
 }
-egg_pys = {} # no more eggs!
 
-default_py = '3.7'
+default_py = "3.8"
 # all the Python versions to be built on linux
-manylinux_pys = '3.8 3.7 2.7 3.5 3.6'
+manylinux_pys = "3.9 3.8 3.7 3.5 3.6"
 
 tmp = "/tmp"
 env_root = os.path.join(tmp, 'envs')
@@ -98,6 +105,12 @@ def cd(path):
     finally:
         os.chdir(cwd)
 
+@task
+def check_pythons(ctx):
+    for version, exe in py_exes.items():
+        print(version, end=": ")
+        ctx.run(f"'{exe}' --version")
+
 @task
 def clone_repo(ctx, reset=False):
     """Clone the repo"""
@@ -215,13 +228,11 @@ def untar(tarball):
     return glob.glob(pjoin(sdist_root, '*'))[0]
 
 @task
-def bdist(ctx, py, wheel=True, egg=False):
+def bdist(ctx, py, wheel=True):
     py = make_env(py, 'wheel')
     cmd = [py, 'setup.py']
     if wheel:
         cmd.append('bdist_wheel')
-    if egg:
-        cmd.append('bdist_egg')
     cmd.append('--zmq=bundled')
 
     run(cmd)
@@ -274,7 +285,7 @@ def release(ctx, vs, upload=False):
         if not os.path.exists(path):
             raise ValueError("Need %s at %s" % (v, path))
 
-    # start from scrach with clone and envs
+    # start from scratch with clone and envs
     clone_repo(ctx, reset=True)
     if os.path.exists(env_root):
         shutil.rmtree(env_root)
@@ -283,7 +294,7 @@ def release(ctx, vs, upload=False):
 
     with cd(path):
         for v in py_exes:
-            bdist(ctx, v, wheel=True, egg=(v in egg_pys))
+            bdist(ctx, v, wheel=True)
         if upload:
             py = make_env(default_py, 'twine')
             run(['twine', 'upload', 'dist/*'])
@@ -291,7 +302,7 @@ def release(ctx, vs, upload=False):
     manylinux(ctx, vs, upload=upload)
     if upload:
         print("When AppVeyor finished building, upload artifacts with:")
-        print("  invoke appveyor_artifacts {} --upload".format(vs))
+        print("  invoke appveyor-artifacts {} --upload".format(vs))
 
 
 _appveyor_api = 'https://ci.appveyor.com/api'
@@ -335,8 +346,7 @@ def appveyor_artifacts(ctx, vs, dest='win-dist', upload=False):
                 f.write(chunk)
     if upload:
         py = make_env(default_py, 'twine')
-        run(['twine', 'upload', '{}/*'.format(dest)])
+        run(['twine', 'upload', '--skip-existing', '{}/*'.format(dest)])
     else:
         print("You can now upload these wheels with: ")
         print("  twine upload {}/*".format(dest))
-
