@@ -1,29 +1,19 @@
 # coding: utf-8
 """zmq Context class"""
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2013 Felipe Cruz
-#
-#  This file is part of pyzmq
-#
-#  Distributed under the terms of the New BSD License.  The full license is in
-#  the file COPYING.BSD, distributed as part of this software.
-#-----------------------------------------------------------------------------
-
-import weakref
+# Copyright (C) PyZMQ Developers
+# Distributed under the terms of the Modified BSD License.
 
 from ._cffi import C, ffi
 
-from .socket import *
-from .constants import *
+from .constants import EINVAL, IO_THREADS, LINGER
 
-from zmq.error import ZMQError, _check_rc
+from zmq.error import ZMQError, InterruptedSystemCall, _check_rc
 
 class Context(object):
     _zmq_ctx = None
     _iothreads = None
     _closed = None
-    _sockets = None
     _shadow = False
 
     def __init__(self, io_threads=1, shadow=None):
@@ -42,7 +32,6 @@ class Context(object):
         if not shadow:
             C.zmq_ctx_set(self._zmq_ctx, IO_THREADS, io_threads)
         self._closed = False
-        self._sockets = set()
     
     @property
     def underlying(self):
@@ -53,18 +42,9 @@ class Context(object):
     def closed(self):
         return self._closed
 
-    def _add_socket(self, socket):
-        ref = weakref.ref(socket)
-        self._sockets.add(ref)
-        return ref
-
-    def _rm_socket(self, ref):
-        if ref in self._sockets:
-            self._sockets.remove(ref)
-
     def set(self, option, value):
         """set a context option
-        
+
         see zmq_ctx_set
         """
         rc = C.zmq_ctx_set(self._zmq_ctx, option, value)
@@ -72,35 +52,26 @@ class Context(object):
 
     def get(self, option):
         """get context option
-        
+
         see zmq_ctx_get
         """
         rc = C.zmq_ctx_get(self._zmq_ctx, option)
-        _check_rc(rc)
+        _check_rc(rc, error_without_errno=False)
         return rc
 
     def term(self):
         if self.closed:
             return
 
-        C.zmq_ctx_destroy(self._zmq_ctx)
+        rc = C.zmq_ctx_destroy(self._zmq_ctx)
+        try:
+            _check_rc(rc)
+        except InterruptedSystemCall:
+            # ignore interrupted term
+            # see PEP 475 notes about close & EINTR for why
+            pass
 
         self._zmq_ctx = None
         self._closed = True
 
-    def destroy(self, linger=None):
-        if self.closed:
-            return
-
-        sockets = self._sockets
-        self._sockets = set()
-        for s in sockets:
-            s = s()
-            if s and not s.closed:
-                if linger:
-                    s.setsockopt(LINGER, linger)
-                s.close()
-        
-        self.term()
-
 __all__ = ['Context']
