@@ -3,6 +3,8 @@
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
+from errno import EINTR
+
 
 class ZMQBaseError(Exception):
     """Base exception class for 0MQ errors in Python."""
@@ -56,7 +58,7 @@ class ZMQError(ZMQBaseError):
         return self.strerror
     
     def __repr__(self):
-        return "ZMQError('%s')"%self.strerror
+        return "%s('%s')" % (self.__class__.__name__, str(self))
 
 
 class ZMQBindError(ZMQBaseError):
@@ -84,7 +86,9 @@ class ContextTerminated(ZMQError):
     
     .. versionadded:: 13.0
     """
-    pass
+    def __init__(self, errno='ignored', msg='ignored'):
+        from zmq import ETERM
+        super(ContextTerminated, self).__init__(ETERM)
 
 
 class Again(ZMQError):
@@ -92,20 +96,49 @@ class Again(ZMQError):
     
     .. versionadded:: 13.0
     """
-    pass
+
+    def __init__(self, errno='ignored', msg='ignored'):
+        from zmq import EAGAIN
+        super(Again, self).__init__(EAGAIN)
 
 
-def _check_rc(rc, errno=None):
+try:
+    InterruptedError
+except NameError:
+    InterruptedError = OSError
+
+class InterruptedSystemCall(ZMQError, InterruptedError):
+    """Wrapper for EINTR
+    
+    This exception should be caught internally in pyzmq
+    to retry system calls, and not propagate to the user.
+    
+    .. versionadded:: 14.7
+    """
+
+    def __init__(self, errno='ignored', msg='ignored'):
+        super(InterruptedSystemCall, self).__init__(EINTR)
+
+    def __str__(self):
+        s = super(InterruptedSystemCall, self).__str__()
+        return s + ": This call should have been retried. Please report this to pyzmq."
+
+
+def _check_rc(rc, errno=None, error_without_errno=True):
     """internal utility for checking zmq return condition
     
     and raising the appropriate Exception class
     """
-    if rc < 0:
-        from zmq.backend import zmq_errno
+    if rc == -1:
         if errno is None:
+            from zmq.backend import zmq_errno
             errno = zmq_errno()
+        if errno == 0 and not error_without_errno:
+            return
         from zmq import EAGAIN, ETERM
-        if errno == EAGAIN:
+        if errno == EINTR:
+            raise InterruptedSystemCall(errno)
+        elif errno == EAGAIN:
             raise Again(errno)
         elif errno == ETERM:
             raise ContextTerminated(errno)
@@ -159,6 +192,7 @@ __all__ = [
     'ZMQError',
     'NotDone',
     'ContextTerminated',
+    'InterruptedSystemCall',
     'Again',
     'ZMQVersionError',
 ]
