@@ -18,24 +18,11 @@ PUB socket.
 Code adapted from StarCluster:
 
     http://github.com/jtriley/StarCluster/blob/master/starcluster/logger.py
-
-Authors
--------
-* Min RK
 """
 
-#-----------------------------------------------------------------------------
-#  Copyright (c) 2010 Brian Granger, Min Ragan-Kelley
-#
-#  This file is part of pyzmq
-#
-#  Distributed under the terms of the New BSD License.  The full license is in
-#  the file COPYING.BSD, distributed as part of this software.
-#-----------------------------------------------------------------------------
+# Copyright (C) PyZMQ Developers
+# Distributed under the terms of the Modified BSD License.
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
 
 import logging
 from logging import INFO, DEBUG, WARN, ERROR, FATAL
@@ -43,9 +30,6 @@ from logging import INFO, DEBUG, WARN, ERROR, FATAL
 import zmq
 from zmq.utils.strtypes import bytes, unicode, cast_bytes
 
-#-----------------------------------------------------------------------------
-# Code
-#-----------------------------------------------------------------------------
 
 TOPIC_DELIM="::" # delimiter for splitting topics on the receiving end.
 
@@ -72,22 +56,23 @@ class PUBHandler(logging.Handler):
     (DEBUG,INFO,etc.), followed by any additional subtopics specified in the
     message by: log.debug("subtopic.subsub::the real message")
     """
-    root_topic=""
+
     socket = None
     
-    formatters = {
-        logging.DEBUG: logging.Formatter(
-        "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n"),
-        logging.INFO: logging.Formatter("%(message)s\n"),
-        logging.WARN: logging.Formatter(
-        "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n"),
-        logging.ERROR: logging.Formatter(
-        "%(levelname)s %(filename)s:%(lineno)d - %(message)s - %(exc_info)s\n"),
-        logging.CRITICAL: logging.Formatter(
-        "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n")}
     
-    def __init__(self, interface_or_socket, context=None):
+    def __init__(self, interface_or_socket, context=None, root_topic=''):
         logging.Handler.__init__(self)
+        self._root_topic = root_topic
+        self.formatters = {
+            logging.DEBUG: logging.Formatter(
+            "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n"),
+            logging.INFO: logging.Formatter("%(message)s\n"),
+            logging.WARN: logging.Formatter(
+            "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n"),
+            logging.ERROR: logging.Formatter(
+            "%(levelname)s %(filename)s:%(lineno)d - %(message)s - %(exc_info)s\n"),
+            logging.CRITICAL: logging.Formatter(
+            "%(levelname)s %(filename)s:%(lineno)d - %(message)s\n")}
         if isinstance(interface_or_socket, zmq.Socket):
             self.socket = interface_or_socket
             self.ctx = self.socket.context
@@ -96,6 +81,41 @@ class PUBHandler(logging.Handler):
             self.socket = self.ctx.socket(zmq.PUB)
             self.socket.bind(interface_or_socket)
 
+    @property
+    def root_topic(self):
+        return self._root_topic
+
+    @root_topic.setter
+    def root_topic(self, value):
+        self.setRootTopic(value)
+
+    def setRootTopic(self, root_topic):
+        """Set the root topic for this handler.
+
+        This value is prepended to all messages published by this handler, and it
+        defaults to the empty string ''. When you subscribe to this socket, you must
+        set your subscription to an empty string, or to at least the first letter of
+        the binary representation of this string to ensure you receive any messages
+        from this handler.
+
+        If you use the default empty string root topic, messages will begin with
+        the binary representation of the log level string (INFO, WARN, etc.).
+        Note that ZMQ SUB sockets can have multiple subscriptions.
+        """
+        self._root_topic = root_topic
+
+    def setFormatter(self, fmt, level=logging.NOTSET):
+        """Set the Formatter for this handler.
+
+        If no level is provided, the same format is used for all levels. This
+        will overwrite all selective formatters set in the object constructor.
+        """
+        if level==logging.NOTSET:
+            for fmt_level in self.formatters.keys():
+                self.formatters[fmt_level] = fmt
+        else:
+            self.formatters[level] = fmt
+
     def format(self,record):
         """Format a record."""
         return self.formatters[record.levelno].format(record)
