@@ -1,17 +1,8 @@
 """0MQ polling related functions and classes."""
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2013 Brian Granger, Min Ragan-Kelley
-#
-#  This file is part of pyzmq
-#
-#  Distributed under the terms of the New BSD License.  The full license is in
-#  the file COPYING.BSD, distributed as part of this software.
-#-----------------------------------------------------------------------------
+# Copyright (C) PyZMQ Developers
+# Distributed under the terms of the Modified BSD License.
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
 
 import zmq
 from zmq.backend import zmq_poll
@@ -85,23 +76,25 @@ class Poller(object):
 
     def poll(self, timeout=None):
         """Poll the registered 0MQ or native fds for I/O.
+        
+        If there are currently events ready to be processed, this function will return immediately.
+        Otherwise, this function will return as soon the first event is available or after timeout 
+        milliseconds have elapsed.
 
         Parameters
         ----------
         timeout : float, int
             The timeout in milliseconds. If None, no `timeout` (infinite). This
-            is in milliseconds to be compatible with ``select.poll()``. The
-            underlying zmq_poll uses microseconds and we convert to that in
-            this function.
-        
+            is in milliseconds to be compatible with ``select.poll()``.
+
         Returns
         -------
         events : list of tuples
             The list of events that are ready to be processed.
-            This is a list of tuples of the form ``(socket, event)``, where the 0MQ Socket
+            This is a list of tuples of the form ``(socket, event_mask)``, where the 0MQ Socket
             or integer fd is the first element, and the poll event mask (POLLIN, POLLOUT) is the second.
             It is common to call ``events = dict(poller.poll())``,
-            which turns the list of tuples into a mapping of ``socket : event``.
+            which turns the list of tuples into a mapping of ``socket : event_mask``.
         """
         if timeout is None or timeout < 0:
             timeout = -1
@@ -121,8 +114,7 @@ def select(rlist, wlist, xlist, timeout=None):
     ----------
     timeout : float, int, optional
         The timeout in seconds. If None, no timeout (infinite). This is in seconds to be
-        compatible with ``select.select()``. The underlying zmq_poll uses microseconds
-        and we convert to that in this function.
+        compatible with ``select.select()``.
     rlist : list of sockets/FDs
         sockets/FDs to be polled for read events
     wlist : list of sockets/FDs
