@@ -3,6 +3,7 @@
 
 import copy
 import gc
+import os
 import sys
 import time
 from threading import Thread, Event
@@ -10,6 +11,10 @@ try:
     from queue import Queue
 except ImportError:
     from Queue import Queue
+try:
+    from unittest import mock
+except ImportError:
+    mock = None
 
 from pytest import mark
 
@@ -35,13 +40,13 @@ class TestContext(BaseZMQTestCase):
 
     def test_init(self):
         c1 = self.Context()
-        self.assert_(isinstance(c1, self.Context))
+        self.assertTrue(isinstance(c1, self.Context))
         del c1
         c2 = self.Context()
-        self.assert_(isinstance(c2, self.Context))
+        self.assertTrue(isinstance(c2, self.Context))
         del c2
         c3 = self.Context()
-        self.assert_(isinstance(c3, self.Context))
+        self.assertTrue(isinstance(c3, self.Context))
         del c3
 
     def test_dir(self):
@@ -51,19 +56,24 @@ class TestContext(BaseZMQTestCase):
             self.assertTrue('IO_THREADS' in dir(ctx))
         ctx.term()
 
+    @mark.skipif(mock is None, reason="requires unittest.mock")
+    def test_mockable(self):
+        m = mock.Mock(spec=self.context)
+
+
     def test_term(self):
         c = self.Context()
         c.term()
-        self.assert_(c.closed)
-    
+        self.assertTrue(c.closed)
+
     def test_context_manager(self):
         with self.Context() as c:
             pass
-        self.assert_(c.closed)
-    
+        self.assertTrue(c.closed)
+
     def test_fail_init(self):
         self.assertRaisesErrno(zmq.EINVAL, self.Context, -1)
-    
+
     def test_term_hang(self):
         rep,req = self.create_bound_pair(zmq.ROUTER, zmq.DEALER)
         req.setsockopt(zmq.LINGER, 0)
@@ -71,7 +81,7 @@ class TestContext(BaseZMQTestCase):
         req.close()
         rep.close()
         self.context.term()
-    
+
     def test_instance(self):
         ctx = self.Context.instance()
         c2 = self.Context.instance(io_threads=2)
@@ -83,6 +93,34 @@ class TestContext(BaseZMQTestCase):
         self.assertFalse(c3.closed)
         self.assertTrue(c3 is c4)
 
+    def test_instance_subclass_first(self):
+        self.context.term()
+        class SubContext(zmq.Context):
+            pass
+        sctx = SubContext.instance()
+        ctx = zmq.Context.instance()
+        ctx.term()
+        sctx.term()
+        assert type(ctx) is zmq.Context
+        assert type(sctx) is SubContext
+
+    def test_instance_subclass_second(self):
+        self.context.term()
+        class SubContextInherit(zmq.Context):
+            pass
+        class SubContextNoInherit(zmq.Context):
+            _instance = None
+            pass
+        ctx = zmq.Context.instance()
+        sctx = SubContextInherit.instance()
+        sctx2 = SubContextNoInherit.instance()
+        ctx.term()
+        sctx.term()
+        sctx2.term()
+        assert type(ctx) is zmq.Context
+        assert type(sctx) is zmq.Context
+        assert type(sctx2) is SubContextNoInherit
+
     def test_instance_threadsafe(self):
         self.context.term() # clear default context
 
@@ -274,8 +312,8 @@ class TestContext(BaseZMQTestCase):
         c2 = copy.copy(c1)
         c2b = copy.deepcopy(c1)
         c3 = copy.deepcopy(c2)
-        self.assert_(c2._shadow)
-        self.assert_(c3._shadow)
+        self.assertTrue(c2._shadow)
+        self.assertTrue(c3._shadow)
         self.assertEqual(c1.underlying, c2.underlying)
         self.assertEqual(c1.underlying, c3.underlying)
         self.assertEqual(c1.underlying, c2b.underlying)
@@ -299,13 +337,13 @@ class TestContext(BaseZMQTestCase):
         ctx.term()
         self.assertRaisesErrno(zmq.EFAULT, ctx2.socket, zmq.PUB)
         del ctx2
-    
+
     def test_shadow_pyczmq(self):
         try:
             from pyczmq import zctx, zsocket, zstr
         except Exception:
             raise SkipTest("Requires pyczmq")
-        
+
         ctx = zctx.new()
         a = zsocket.new(ctx, zmq.PUSH)
         zsocket.bind(a, "inproc://a")
@@ -317,6 +355,33 @@ class TestContext(BaseZMQTestCase):
         self.assertEqual(rcvd, b'hi')
         b.close()
 
+    @mark.skipif(
+        sys.platform.startswith('win'),
+        reason='No fork on Windows')
+    def test_fork_instance(self):
+        ctx = self.Context.instance()
+        parent_ctx_id = id(ctx)
+        r_fd, w_fd = os.pipe()
+        reader = os.fdopen(r_fd, 'r')
+        child_pid = os.fork()
+        if child_pid == 0:
+            ctx = self.Context.instance()
+            writer = os.fdopen(w_fd, 'w')
+            child_ctx_id = id(ctx)
+            ctx.term()
+            writer.write(str(child_ctx_id) + "\n")
+            writer.flush()
+            writer.close()
+            os._exit(0)
+        else:
+            os.close(w_fd)
+
+        child_id_s = reader.readline()
+        reader.close()
+        assert child_id_s
+        assert int(child_id_s) != parent_ctx_id
+        ctx.term()
+
 
 if False: # disable green context tests
     class TestContextGreen(GreenTest, TestContext):
