@@ -1,6 +1,11 @@
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
+from __future__ import absolute_import
+try:
+    import asyncio
+except ImportError:
+    asyncio = None
 import time
 import os
 import threading
@@ -9,13 +14,21 @@ import pytest
 
 import zmq
 from zmq.tests import BaseZMQTestCase, have_gevent
-from zmq.eventloop import ioloop
 try:
     from tornado.ioloop import IOLoop as BaseIOLoop
+    from zmq.eventloop import ioloop
+    _tornado = True
 except ImportError:
-    from zmq.eventloop.minitornado.ioloop import IOLoop as BaseIOLoop
+    _tornado = False
 
 
+# tornado 5 with asyncio disables custom IOLoop implementations
+t5asyncio = False
+if _tornado:
+    import tornado
+    if tornado.version_info >= (5,) and asyncio:
+        t5asyncio = True
+
 def printer():
     os.system("say hello")
     raise Exception
@@ -29,14 +42,14 @@ class Delay(threading.Thread):
         self.aborted=False
         self.cond=threading.Condition()
         super(Delay, self).__init__()
-    
+
     def run(self):
         self.cond.acquire()
         self.cond.wait(self.delay)
         self.cond.release()
         if not self.aborted:
             self.f()
-    
+
     def abort(self):
         self.aborted=True
         self.cond.acquire()
@@ -45,6 +58,15 @@ class Delay(threading.Thread):
 
 
 class TestIOLoop(BaseZMQTestCase):
+    if _tornado:
+        IOLoop = ioloop.IOLoop
+
+    def setUp(self):
+        if not _tornado:
+            pytest.skip("tornado required")
+        super(TestIOLoop, self).setUp()
+        if asyncio:
+            asyncio.set_event_loop(asyncio.new_event_loop())
 
     def tearDown(self):
         super(TestIOLoop, self).tearDown()
@@ -53,58 +75,39 @@ class TestIOLoop(BaseZMQTestCase):
 
     def test_simple(self):
         """simple IOLoop creation test"""
-        loop = ioloop.IOLoop()
-        dc = ioloop.PeriodicCallback(loop.stop, 200, loop)
-        pc = ioloop.PeriodicCallback(lambda : None, 10, loop)
+        loop = self.IOLoop()
+        loop.make_current()
+        dc = ioloop.PeriodicCallback(loop.stop, 200)
+        pc = ioloop.PeriodicCallback(lambda : None, 10)
         pc.start()
         dc.start()
         t = Delay(loop.stop,1)
         t.start()
         loop.start()
-        if t.isAlive():
+        if t.is_alive():
             t.abort()
         else:
             self.fail("IOLoop failed to exit")
-    
-    def test_poller_events(self):
-        """Tornado poller implementation maps events correctly"""
-        req,rep = self.create_bound_pair(zmq.REQ, zmq.REP)
-        poller = ioloop.ZMQPoller()
-        poller.register(req, ioloop.IOLoop.READ)
-        poller.register(rep, ioloop.IOLoop.READ)
-        events = dict(poller.poll(0))
-        self.assertEqual(events.get(rep), None)
-        self.assertEqual(events.get(req), None)
-        
-        poller.register(req, ioloop.IOLoop.WRITE)
-        poller.register(rep, ioloop.IOLoop.WRITE)
-        events = dict(poller.poll(1))
-        self.assertEqual(events.get(req), ioloop.IOLoop.WRITE)
-        self.assertEqual(events.get(rep), None)
-        
-        poller.register(rep, ioloop.IOLoop.READ)
-        req.send(b'hi')
-        events = dict(poller.poll(1))
-        self.assertEqual(events.get(rep), ioloop.IOLoop.READ)
-        self.assertEqual(events.get(req), None)
 
     def test_instance(self):
-        """Green IOLoop.instance returns the right object"""
-        loop = ioloop.IOLoop.instance()
-        assert isinstance(loop, ioloop.IOLoop)
+        """IOLoop.instance returns the right object"""
+        loop = self.IOLoop.instance()
+        if not t5asyncio:
+            assert isinstance(loop, self.IOLoop)
         base_loop = BaseIOLoop.instance()
         assert base_loop is loop
 
     def test_current(self):
-        """Green IOLoop.current returns the right object"""
+        """IOLoop.current returns the right object"""
         loop = ioloop.IOLoop.current()
-        assert isinstance(loop, ioloop.IOLoop)
+        if not t5asyncio:
+            assert isinstance(loop, self.IOLoop)
         base_loop = BaseIOLoop.current()
         assert base_loop is loop
 
     def test_close_all(self):
         """Test close(all_fds=True)"""
-        loop = ioloop.IOLoop.instance()
+        loop = self.IOLoop.current()
         req,rep = self.create_bound_pair(zmq.REQ, zmq.REP)
         loop.add_handler(req, lambda msg: msg, ioloop.IOLoop.READ)
         loop.add_handler(rep, lambda msg: msg, ioloop.IOLoop.READ)
@@ -113,23 +116,26 @@ class TestIOLoop(BaseZMQTestCase):
         loop.close(all_fds=True)
         self.assertEqual(req.closed, True)
         self.assertEqual(rep.closed, True)
-        
 
-if have_gevent:
+
+if have_gevent and _tornado:
     import zmq.green.eventloop.ioloop as green_ioloop
-    
-    class TestIOLoopGreen(BaseZMQTestCase):
-        def test_instance(self):
+
+    class TestIOLoopGreen(TestIOLoop):
+        IOLoop = green_ioloop.IOLoop
+        def xtest_instance(self):
             """Green IOLoop.instance returns the right object"""
-            loop = green_ioloop.IOLoop.instance()
-            assert isinstance(loop, green_ioloop.IOLoop)
+            loop = self.IOLoop.instance()
+            if not t5asyncio:
+                assert isinstance(loop, self.IOLoop)
             base_loop = BaseIOLoop.instance()
             assert base_loop is loop
-    
-        def test_current(self):
+
+        def xtest_current(self):
             """Green IOLoop.current returns the right object"""
-            loop = green_ioloop.IOLoop.current()
-            assert isinstance(loop, green_ioloop.IOLoop)
+            loop = self.IOLoop.current()
+            if not t5asyncio:
+                assert isinstance(loop, self.IOLoop)
             base_loop = BaseIOLoop.current()
             assert base_loop is loop
-    
+
